@@ -8,7 +8,6 @@ import (
 	"github.com/brimdata/zed/cmd/zed/root"
 	"github.com/brimdata/zed/lakeparse"
 	"github.com/brimdata/zed/pkg/charm"
-	"github.com/brimdata/zed/pkg/rlimit"
 )
 
 var Cmd = &charm.Spec{
@@ -38,9 +37,6 @@ func (c *Command) Run(args []string) error {
 	if err != nil {
 		return err
 	}
-	if _, err := rlimit.RaiseOpenFilesLimit(); err != nil {
-		return err
-	}
 	defer cleanup()
 	ids, err := lakeparse.ParseIDs(args)
 	if err != nil {
