@@ -5,12 +5,9 @@ import (
 	"flag"
 	"fmt"
 
-	"github.com/brimdata/zed/cli"
-	"github.com/brimdata/zed/cli/lakeflags"
 	"github.com/brimdata/zed/cmd/zed/root"
 	"github.com/brimdata/zed/lake/api"
 	"github.com/brimdata/zed/pkg/charm"
-	"github.com/brimdata/zed/pkg/storage"
 )
 
 var Cmd = &charm.Spec{
@@ -25,15 +22,10 @@ var Cmd = &charm.Spec{
 
 type Command struct {
 	*root.Command
-	cli.LakeFlags
-	lakeFlags lakeflags.Flags
 }
 
 func New(parent charm.Command, f *flag.FlagSet) (charm.Command, error) {
-	c := &Command{Command: parent.(*root.Command)}
-	c.LakeFlags.SetFlags(f)
-	c.lakeFlags.SetFlags(f)
-	return c, nil
+	return &Command{Command: parent.(*root.Command)}, nil
 }
 
 func (c *Command) Run(args []string) error {
@@ -44,24 +36,20 @@ func (c *Command) Run(args []string) error {
 	defer cleanup()
 	var path string
 	if len(args) == 0 {
-		path = c.Lake
+		path = c.LakeFlags.Lake
 	} else if len(args) == 1 {
 		path = args[0]
 	}
 	if path == "" {
 		return errors.New("single lake path argument required")
 	}
-	lakePath, err := storage.ParseURI(path)
-	if err != nil {
-		return err
-	}
-	if api.IsLakeService(lakePath) {
+	if api.IsLakeService(path) {
 		return fmt.Errorf("init command not valid on remote lake")
 	}
-	if _, err := api.CreateLocalLake(ctx, lakePath); err != nil {
+	if _, err := api.CreateLocalLake(ctx, path); err != nil {
 		return err
 	}
-	if !c.lakeFlags.Quiet {
+	if !c.LakeFlags.Quiet {
 		fmt.Printf("lake created: %s\n", path)
 	}
 	return nil
