@@ -11,16 +11,14 @@ import (
 	"runtime"
 
 	"github.com/brimdata/zed/cli"
+	"github.com/brimdata/zed/cli/logflags"
 	"github.com/brimdata/zed/cmd/zed/root"
 	"github.com/brimdata/zed/lake/api"
 	"github.com/brimdata/zed/pkg/charm"
 	"github.com/brimdata/zed/pkg/fs"
 	"github.com/brimdata/zed/pkg/httpd"
-	"github.com/brimdata/zed/pkg/rlimit"
 	"github.com/brimdata/zed/service"
-	"github.com/brimdata/zed/service/logger"
 	"go.uber.org/zap"
-	"go.uber.org/zap/zapcore"
 )
 
 var Cmd = &charm.Spec{
@@ -38,14 +36,12 @@ Requests may be issued to this service via the "zed api" command.
 
 type Command struct {
 	*root.Command
-	conf    service.Config
-	logger  *zap.Logger
-	logconf logger.Config
+	conf     service.Config
+	logflags logflags.Flags
 
 	// brimfd is a file descriptor passed through by brim desktop. If set the
 	// command will exit if the fd is closed.
 	brimfd          int
-	devMode         bool
 	listenAddr      string
 	portFile        string
 	rootContentFile string
@@ -55,13 +51,9 @@ func New(parent charm.Command, f *flag.FlagSet) (charm.Command, error) {
 	c := &Command{Command: parent.(*root.Command)}
 	c.conf.Auth.SetFlags(f)
 	c.conf.Version = cli.Version
+	c.logflags.SetFlags(f)
 	f.IntVar(&c.brimfd, "brimfd", -1, "pipe read fd passed by brim to signal brim closure")
-	f.BoolVar(&c.devMode, "dev", false, "run in development mode")
 	f.StringVar(&c.listenAddr, "l", ":9867", "[addr]:port to listen on")
-	f.Var(&c.logconf.Level, "log.level", "logging level")
-	f.StringVar(&c.logconf.Path, "log.path", "stderr", "logging level")
-	c.logconf.Mode = logger.FileModeTruncate
-	f.Var(&c.logconf.Mode, "log.filemode", "logger file write mode (values: append, truncate, rotate)")
 	f.StringVar(&c.portFile, "portfile", "", "write listen port to file")
 	f.StringVar(&c.rootContentFile, "rootcontentfile", "", "file to serve for GET /")
 	return c, nil
@@ -92,21 +84,17 @@ func (c *Command) Run(args []string) error {
 		defer f.Close()
 		c.conf.RootContent = f
 	}
-	if err := c.initLogger(); err != nil {
+	logger, err := c.logflags.Open()
+	if err != nil {
 		return err
 	}
-	defer c.logger.Sync()
-	openFilesLimit, err := rlimit.RaiseOpenFilesLimit()
-	if err != nil {
-		c.logger.Warn("Raising open files limit failed", zap.Error(err))
-	}
-	c.conf.Logger.Info("Open files limit raised", zap.Uint64("limit", openFilesLimit))
-
+	defer logger.Sync()
 	if c.brimfd != -1 {
-		if ctx, err = c.watchBrimFd(ctx); err != nil {
+		if ctx, err = c.watchBrimFd(ctx, logger); err != nil {
 			return err
 		}
 	}
+	c.conf.Logger = logger
 	core, err := service.NewCore(ctx, c.conf)
 	if err != nil {
 		return err
@@ -116,10 +104,10 @@ func (c *Command) Run(args []string) error {
 	signal.Notify(sigch, os.Interrupt)
 	go func() {
 		sig := <-sigch
-		c.logger.Info("Signal received", zap.Stringer("signal", sig))
+		logger.Info("Signal received", zap.Stringer("signal", sig))
 	}()
 	srv := httpd.New(c.listenAddr, core)
-	srv.SetLogger(c.logger.Named("httpd"))
+	srv.SetLogger(logger.Named("httpd"))
 	if err := srv.Start(ctx); err != nil {
 		return err
 	}
@@ -131,37 +119,21 @@ func (c *Command) Run(args []string) error {
 	return srv.Wait()
 }
 
-func (c *Command) watchBrimFd(ctx context.Context) (context.Context, error) {
+func (c *Command) watchBrimFd(ctx context.Context, logger *zap.Logger) (context.Context, error) {
 	if runtime.GOOS == "windows" {
 		return nil, errors.New("flag -brimfd not applicable to windows")
 	}
 	f := os.NewFile(uintptr(c.brimfd), "brimfd")
-	c.logger.Info("Listening to brim process pipe", zap.String("fd", f.Name()))
+	logger.Info("Listening to brim process pipe", zap.String("fd", f.Name()))
 	ctx, cancel := context.WithCancel(ctx)
 	go func() {
 		io.Copy(io.Discard, f)
-		c.logger.Info("Brim fd closed, shutting down")
+		logger.Info("Brim fd closed, shutting down")
 		cancel()
 	}()
 	return ctx, nil
 }
 
-func (c *Command) initLogger() error {
-	core, err := logger.NewCore(c.logconf)
-	if err != nil {
-		return err
-	}
-	opts := []zap.Option{zap.AddStacktrace(zapcore.WarnLevel)}
-	// If the development mode is on, calls to logger.DPanic will cause a panic
-	// whereas in production would result in an error.
-	if c.devMode {
-		opts = append(opts, zap.Development())
-	}
-	c.logger = zap.New(core, opts...)
-	c.conf.Logger = c.logger
-	return nil
-}
-
 func (c *Command) writePortFile(addr string) error {
 	_, port, err := net.SplitHostPort(addr)
 	if err != nil {
