@@ -25,9 +25,9 @@ import (
 // various type checks and coercions that determine different computational
 // outcomes based on type.  There is nothing here that optimizes analytics
 // for native machine types; these optimizations (will) happen in the pushdown
-// predicate processing engine in the zst columnar scanner.
+// predicate processing engine in the VNG columnar scanner.
 //
-// Eventually, we will optimize this zst "fast path" by dynamically
+// Eventually, we will optimize this VNG "fast path" by dynamically
 // generating byte codes (which can in turn be JIT assembled into machine code)
 // for each zng TypeRecord encountered.  Once you know the TypeRecord,
 // you can generate code using strong typing just as an OLAP system does
@@ -335,10 +335,16 @@ func (b *Builder) compileCall(call dag.Call) (expr.Evaluator, error) {
 	if isShaperFunc(call.Name) {
 		return b.compileShaper(call)
 	}
-	nargs := len(call.Args)
-	fn, path, err := function.New(b.zctx(), call.Name, nargs)
-	if err != nil {
-		return nil, fmt.Errorf("%s(): %w", call.Name, err)
+	var path field.Path
+	// First check if call is to a user defined function, otherwise check for
+	// builtin function.
+	fn, ok := b.funcs[call.Name]
+	if !ok {
+		var err error
+		fn, path, err = function.New(b.zctx(), call.Name, len(call.Args))
+		if err != nil {
+			return nil, fmt.Errorf("%s(): %w", call.Name, err)
+		}
 	}
 	args := call.Args
 	if path != nil {
