@@ -20,11 +20,11 @@ func (i *anyCompiler) NewQuery(pctx *op.Context, o ast.Op, readers []zio.Reader)
 	if len(readers) != 1 {
 		return nil, fmt.Errorf("NewQuery: Zed program expected %d readers", len(readers))
 	}
-	return i.CompileWithOrderDeprecated(pctx, o, readers[0], order.Layout{})
+	return CompileWithLayout(pctx, o, readers[0], order.Layout{})
 }
 
-//XXX currently used only by group-by test, need to deprecate
-func (*anyCompiler) CompileWithOrderDeprecated(pctx *op.Context, o ast.Op, r zio.Reader, layout order.Layout) (*runtime.Query, error) {
+// XXX currently used only by group-by test, need to deprecate
+func CompileWithLayout(pctx *op.Context, o ast.Op, r zio.Reader, layout order.Layout) (*runtime.Query, error) {
 	job, err := NewJob(pctx, o, data.NewSource(nil, nil), nil)
 	if err != nil {
 		return nil, err
@@ -41,3 +41,7 @@ func (*anyCompiler) CompileWithOrderDeprecated(pctx *op.Context, o ast.Op, r zio
 func (*anyCompiler) NewLakeQuery(pctx *op.Context, program ast.Op, parallelism int, head *lakeparse.Commitish) (*runtime.Query, error) {
 	panic("NewLakeQuery called on compiler.anyCompiler")
 }
+
+func (*anyCompiler) NewLakeDeleteQuery(pctx *op.Context, program ast.Op, head *lakeparse.Commitish) (*runtime.DeleteQuery, error) {
+	panic("NewLakeDeleteQuery called on compiler.anyCompiler")
+}
