@@ -11,7 +11,6 @@ import (
 	"github.com/brimdata/zed/lake/index"
 	"github.com/brimdata/zed/lakeparse"
 	"github.com/brimdata/zed/order"
-	"github.com/brimdata/zed/pkg/rlimit"
 	"github.com/brimdata/zed/pkg/storage"
 	"github.com/brimdata/zed/runtime"
 	"github.com/brimdata/zed/runtime/exec"
@@ -106,7 +105,7 @@ func (l *local) Compact(ctx context.Context, poolID ksuid.KSUID, branchName stri
 	if err != nil {
 		return ksuid.Nil, err
 	}
-	return exec.Compact(ctx, pool, branchName, objects, commit.Author, commit.Body, commit.Meta)
+	return exec.Compact(ctx, l.root, pool, branchName, objects, commit.Author, commit.Body, commit.Meta)
 }
 
 func (l *local) AddIndexRules(ctx context.Context, rules []index.Rule) error {
@@ -130,9 +129,6 @@ func (l *local) QueryWithControl(ctx context.Context, head *lakeparse.Commitish,
 	if err != nil {
 		return nil, err
 	}
-	if _, err := rlimit.RaiseOpenFilesLimit(); err != nil {
-		return nil, err
-	}
 	q, err := runtime.CompileLakeQuery(ctx, zed.NewContext(), l.compiler, flowgraph, head, nil)
 	if err != nil {
 		return nil, err
@@ -183,19 +179,23 @@ func (l *local) Delete(ctx context.Context, poolID ksuid.KSUID, branchName strin
 	return commitID, nil
 }
 
-func (l *local) DeleteByPredicate(ctx context.Context, poolID ksuid.KSUID, branchName, src string, commit api.CommitMessage) (ksuid.KSUID, error) {
+func (l *local) DeleteWhere(ctx context.Context, poolID ksuid.KSUID, branchName, src string, commit api.CommitMessage) (ksuid.KSUID, error) {
+	op, err := l.compiler.Parse(src)
+	if err != nil {
+		return ksuid.Nil, err
+	}
 	_, branch, err := l.lookupBranch(ctx, poolID, branchName)
 	if err != nil {
 		return ksuid.Nil, err
 	}
-	return branch.DeleteByPredicate(ctx, l.compiler, src, commit.Author, commit.Body, commit.Meta)
+	return branch.DeleteWhere(ctx, l.compiler, op, commit.Author, commit.Body, commit.Meta)
 }
 
 func (l *local) Revert(ctx context.Context, poolID ksuid.KSUID, branchName string, commitID ksuid.KSUID, message api.CommitMessage) (ksuid.KSUID, error) {
 	return l.root.Revert(ctx, poolID, branchName, commitID, message.Author, message.Body)
 }
 
-func (l *local) ApplyIndexRules(ctx context.Context, name string, poolID ksuid.KSUID, branchName string, inTags []ksuid.KSUID) (ksuid.KSUID, error) {
+func (l *local) ApplyIndexRules(ctx context.Context, ruleRefs []string, poolID ksuid.KSUID, branchName string, inTags []ksuid.KSUID) (ksuid.KSUID, error) {
 	_, branch, err := l.lookupBranch(ctx, poolID, branchName)
 	if err != nil {
 		return ksuid.Nil, err
@@ -204,7 +204,7 @@ func (l *local) ApplyIndexRules(ctx context.Context, name string, poolID ksuid.K
 	if err != nil {
 		return ksuid.Nil, err
 	}
-	rules, err := l.root.LookupIndexRules(ctx, name)
+	rules, err := l.root.LookupIndexRules(ctx, lakeparse.FormatIDs(ruleRefs)...)
 	if err != nil {
 		return ksuid.Nil, err
 	}
@@ -215,19 +215,35 @@ func (l *local) ApplyIndexRules(ctx context.Context, name string, poolID ksuid.K
 	return commit, nil
 }
 
-func (l *local) UpdateIndex(ctx context.Context, names []string, poolID ksuid.KSUID, branchName string) (ksuid.KSUID, error) {
+func (l *local) UpdateIndex(ctx context.Context, ruleRefs []string, poolID ksuid.KSUID, branchName string) (ksuid.KSUID, error) {
 	_, branch, err := l.lookupBranch(ctx, poolID, branchName)
 	if err != nil {
 		return ksuid.Nil, err
 	}
 	var rules []index.Rule
-	if len(names) == 0 {
+	if len(ruleRefs) == 0 {
 		rules, err = l.root.AllIndexRules(ctx)
 	} else {
-		rules, err = l.root.LookupIndexRules(ctx, names...)
+		rules, err = l.root.LookupIndexRules(ctx, lakeparse.FormatIDs(ruleRefs)...)
 	}
 	if err != nil {
 		return ksuid.Nil, err
 	}
 	return branch.UpdateIndex(ctx, l.compiler, rules)
 }
+
+func (l *local) AddVectors(ctx context.Context, poolID ksuid.KSUID, branchName string, ids []ksuid.KSUID, message api.CommitMessage) (ksuid.KSUID, error) {
+	_, branch, err := l.lookupBranch(ctx, poolID, branchName)
+	if err != nil {
+		return ksuid.Nil, err
+	}
+	return branch.AddVectors(ctx, ids, message.Author, message.Body)
+}
+
+func (l *local) DeleteVectors(ctx context.Context, poolID ksuid.KSUID, branchName string, ids []ksuid.KSUID, message api.CommitMessage) (ksuid.KSUID, error) {
+	_, branch, err := l.lookupBranch(ctx, poolID, branchName)
+	if err != nil {
+		return ksuid.Nil, err
+	}
+	return branch.DeleteVectors(ctx, ids, message.Author, message.Body)
+}
