@@ -95,11 +95,14 @@ func (r *remote) RenamePool(ctx context.Context, pool ksuid.KSUID, name string)
 func (r *remote) Load(ctx context.Context, _ *zed.Context, poolID ksuid.KSUID, branchName string, reader zio.Reader, commit api.CommitMessage) (ksuid.KSUID, error) {
 	pr, pw := io.Pipe()
 	go func() {
-		w := zngio.NewWriter(pw)
-		zio.CopyWithContext(ctx, w, reader)
-		w.Close()
+		w := zngio.NewWriter(zio.NopCloser(pw))
+		err := zio.CopyWithContext(ctx, w, reader)
+		if err2 := w.Close(); err == nil {
+			err = err2
+		}
+		pw.CloseWithError(err)
 	}()
-	res, err := r.conn.Load(ctx, poolID, branchName, pr, commit)
+	res, err := r.conn.Load(ctx, poolID, branchName, api.MediaTypeZNG, pr, commit)
 	return res.Commit, err
 }
 
@@ -133,8 +136,8 @@ func (r *remote) Delete(ctx context.Context, poolID ksuid.KSUID, branchName stri
 	return res.Commit, err
 }
 
-func (r *remote) DeleteByPredicate(ctx context.Context, poolID ksuid.KSUID, branchName, src string, commit api.CommitMessage) (ksuid.KSUID, error) {
-	res, err := r.conn.DeleteByPredicate(ctx, poolID, branchName, src, commit)
+func (r *remote) DeleteWhere(ctx context.Context, poolID ksuid.KSUID, branchName, src string, commit api.CommitMessage) (ksuid.KSUID, error) {
+	res, err := r.conn.DeleteWhere(ctx, poolID, branchName, src, commit)
 	return res.Commit, err
 }
 
@@ -147,8 +150,8 @@ func (r *remote) DeleteIndexRules(ctx context.Context, ids []ksuid.KSUID) ([]ind
 	return res.Rules, err
 }
 
-func (r *remote) ApplyIndexRules(ctx context.Context, rule string, poolID ksuid.KSUID, branchName string, inTags []ksuid.KSUID) (ksuid.KSUID, error) {
-	res, err := r.conn.ApplyIndexRules(ctx, poolID, branchName, rule, inTags)
+func (r *remote) ApplyIndexRules(ctx context.Context, rules []string, poolID ksuid.KSUID, branchName string, inTags []ksuid.KSUID) (ksuid.KSUID, error) {
+	res, err := r.conn.ApplyIndexRules(ctx, poolID, branchName, rules, inTags)
 	return res.Commit, err
 }
 
@@ -156,3 +159,11 @@ func (r *remote) UpdateIndex(ctx context.Context, rules []string, poolID ksuid.K
 	res, err := r.conn.UpdateIndex(ctx, poolID, branchName, rules)
 	return res.Commit, err
 }
+
+func (r *remote) AddVectors(ctx context.Context, pool ksuid.KSUID, branch string, objects []ksuid.KSUID, message api.CommitMessage) (ksuid.KSUID, error) {
+	panic("TBD")
+}
+
+func (r *remote) DeleteVectors(ctx context.Context, poolID ksuid.KSUID, branchName string, ids []ksuid.KSUID, message api.CommitMessage) (ksuid.KSUID, error) {
+	panic("TBD")
+}
