@@ -1,10 +1,12 @@
-package index
+package index_test
 
 import (
 	"context"
 	"testing"
 
 	"github.com/brimdata/zed"
+	"github.com/brimdata/zed/compiler"
+	"github.com/brimdata/zed/lake/index"
 	"github.com/brimdata/zed/pkg/storage"
 	"github.com/brimdata/zed/zio"
 	"github.com/segmentio/ksuid"
@@ -13,8 +15,8 @@ import (
 )
 
 func TestWriter(t *testing.T) {
-	r := NewTypeRule("test", zed.TypeInt64)
-	o := Object{Rule: r, ID: ksuid.New()}
+	r := index.NewTypeRule("test", zed.TypeInt64)
+	o := index.Object{Rule: r, ID: ksuid.New()}
 	w := testWriter(t, o)
 	err := zio.Copy(w, babbleReader(t))
 	require.NoError(t, err, "copy error")
@@ -22,8 +24,8 @@ func TestWriter(t *testing.T) {
 }
 
 func TestWriterWriteAfterClose(t *testing.T) {
-	r := NewTypeRule("test", zed.TypeInt64)
-	o := Object{Rule: r, ID: ksuid.New()}
+	r := index.NewTypeRule("test", zed.TypeInt64)
+	o := index.Object{Rule: r, ID: ksuid.New()}
 	w := testWriter(t, o)
 	require.NoError(t, w.Close())
 	err := w.Write(nil)
@@ -32,9 +34,10 @@ func TestWriterWriteAfterClose(t *testing.T) {
 	assert.EqualError(t, err, "index writer closed")
 }
 
-func testWriter(t *testing.T, o Object) *Writer {
+func testWriter(t *testing.T, o index.Object) *index.Writer {
 	path := storage.MustParseURI(t.TempDir())
-	w, err := NewWriter(context.Background(), storage.NewLocalEngine(), path, &o)
+	comp := compiler.NewCompiler()
+	w, err := index.NewWriter(context.Background(), comp, storage.NewLocalEngine(), path, &o)
 	require.NoError(t, err)
 	return w
 }
