@@ -1,6 +1,9 @@
 package seekindex
 
-import "io"
+import (
+	"fmt"
+	"io"
+)
 
 type Range struct {
 	Start, End int64
@@ -31,6 +34,10 @@ func (r Range) IsZero() bool {
 	return r.Start == 0 && r.End == 0
 }
 
+func (r Range) String() string {
+	return fmt.Sprintf("start %d end %d", r.Start, r.End)
+}
+
 func (r Range) Reader(reader io.ReaderAt) (io.Reader, error) {
 	return io.NewSectionReader(reader, r.Start, r.Size()), nil
 }
