@@ -48,7 +48,7 @@ type Writer struct {
 
 // NewWriter creates a zio.Writer compliant writer for writing data to an
 // a data pool presuming the input is not guaranteed to be sorted.
-//XXX we should make another writer that takes sorted input and is a bit
+// XXX we should make another writer that takes sorted input and is a bit
 // more efficient.  This other writer could have different commit triggers
 // to do useful things like paritioning given the context is a rollup.
 func NewWriter(ctx context.Context, zctx *zed.Context, pool *Pool) (*Writer, error) {
@@ -81,7 +81,11 @@ func (w *Writer) Write(rec *zed.Value) error {
 		}
 		return w.ctx.Err()
 	}
-	w.append(rec)
+	// XXX This call leads to a ton of one-off allocations that burden the GC
+	// and slow down import. We should instead copy the raw record bytes into a
+	// recycled buffer and keep around an array of ts + byte-slice structs for
+	// sorting.
+	w.vals = append(w.vals, *rec.Copy())
 	w.memBuffered += int64(len(rec.Bytes))
 	//XXX change name LogSizeThreshold
 	// XXX the previous logic estimated the object size with divide by 2...?!
@@ -91,16 +95,6 @@ func (w *Writer) Write(rec *zed.Value) error {
 	return nil
 }
 
-func (w *Writer) append(val *zed.Value) {
-	n := len(w.vals)
-	if n < cap(w.vals) {
-		w.vals = w.vals[:n+1]
-	} else {
-		w.vals = append(w.vals, zed.Value{})
-	}
-	w.vals[n].CopyFrom(val)
-}
-
 func (w *Writer) flipBuffers() {
 	oldvals, ok := <-w.buffer
 	if !ok {
