@@ -36,32 +36,14 @@ func NewDottedExpr(zctx *zed.Context, f field.Path) Evaluator {
 	return ret
 }
 
-func ValueUnder(val *zed.Value) *zed.Value {
-	typ := val.Type
-	if _, ok := typ.(*zed.TypeNamed); !ok {
-		if _, ok := typ.(*zed.TypeUnion); !ok {
-			// common fast path
-			return val
-		}
-	}
-	bytes := val.Bytes
-	for {
-		typ = zed.TypeUnder(typ)
-		union, ok := typ.(*zed.TypeUnion)
-		if !ok {
-			return &zed.Value{typ, bytes}
-		}
-		typ, bytes = union.SplitZNG(bytes)
-	}
-}
-
 func (d *DotExpr) Eval(ectx Context, this *zed.Value) *zed.Value {
-	rec := d.record.Eval(ectx, this)
-	val := ValueUnder(rec)
+	val := d.record.Eval(ectx, this).Under()
 	if _, ok := val.Type.(*zed.TypeOfType); ok {
 		return d.evalTypeOfType(ectx, val.Bytes)
 	}
-
+	if typ, ok := val.Type.(*zed.TypeMap); ok {
+		return indexMap(d.zctx, ectx, typ, val.Bytes, zed.NewString(d.field))
+	}
 	recType, ok := val.Type.(*zed.TypeRecord)
 	if !ok {
 		return d.zctx.Missing()
