@@ -109,11 +109,11 @@ func (r *recordSpreadExpr) Eval(ectx Context, this *zed.Value) *zed.Value {
 			for _, col := range typ.Columns {
 				c, ok := object[col.Name]
 				if ok {
-					c.value = zed.Value{col.Type, it.Next()}
+					c.value = zed.Value{Type: col.Type, Bytes: it.Next()}
 				} else {
 					c = column{
 						colno: len(object),
-						value: zed.Value{col.Type, it.Next()},
+						value: zed.Value{Type: col.Type, Bytes: it.Next()},
 					}
 				}
 				object[col.Name] = c
@@ -153,7 +153,7 @@ func (r *recordSpreadExpr) update(object map[string]column) {
 		return
 	}
 	for name, field := range object {
-		col := zed.Column{name, field.value.Type}
+		col := zed.Column{Name: name, Type: field.value.Type}
 		if r.columns[field.colno] != col {
 			r.invalidate(object)
 			return
@@ -171,7 +171,7 @@ func (r *recordSpreadExpr) invalidate(object map[string]column) {
 		r.bytes = r.bytes[:n]
 	}
 	for name, field := range object {
-		r.columns[field.colno] = zed.Column{name, field.value.Type}
+		r.columns[field.colno] = zed.Column{Name: name, Type: field.value.Type}
 		r.bytes[field.colno] = field.value.Bytes
 	}
 	r.cache = r.zctx.MustLookupTypeRecord(r.columns)
@@ -284,7 +284,6 @@ func NewMapExpr(zctx *zed.Context, entries []Entry) *MapExpr {
 }
 
 func (m *MapExpr) Eval(ectx Context, this *zed.Value) *zed.Value {
-	m.builder.Reset()
 	m.keys.reset()
 	m.vals.reset()
 	for _, e := range m.entries {
@@ -295,6 +294,7 @@ func (m *MapExpr) Eval(ectx Context, this *zed.Value) *zed.Value {
 		typ := m.zctx.LookupTypeMap(zed.TypeNull, zed.TypeNull)
 		return ectx.NewValue(typ, []byte{})
 	}
+	m.builder.Reset()
 	kIter, vIter := m.keys.iter(m.zctx), m.vals.iter(m.zctx)
 	for !kIter.done() {
 		kIter.appendNext(&m.builder)
@@ -337,6 +337,7 @@ func (c *collectionBuilder) iter(zctx *zed.Context) collectionIter {
 		typ:   unionOf(zctx, c.uniqueTypes),
 		bytes: c.bytes,
 		types: c.types,
+		uniq:  len(c.uniqueTypes),
 	}
 }
 
@@ -344,11 +345,12 @@ type collectionIter struct {
 	typ   zed.Type
 	bytes []zcode.Bytes
 	types []zed.Type
+	uniq  int
 }
 
 func (c *collectionIter) appendNext(b *zcode.Builder) {
-	if union, ok := c.typ.(*zed.TypeUnion); ok {
-		zed.BuildUnion(b, union.Selector(c.types[0]), c.bytes[0])
+	if union, ok := c.typ.(*zed.TypeUnion); ok && c.uniq > 1 {
+		zed.BuildUnion(b, union.TagOf(c.types[0]), c.bytes[0])
 	} else {
 		b.Append(c.bytes[0])
 	}
