@@ -2,6 +2,7 @@ package groupby
 
 import (
 	"context"
+	"encoding/binary"
 	"sync"
 
 	"github.com/brimdata/zed"
@@ -47,7 +48,7 @@ type Aggregator struct {
 	keyExprs       []expr.Evaluator
 	aggRefs        []expr.Evaluator
 	aggs           []*expr.Aggregator
-	builder        *zed.ColumnBuilder
+	builder        *zed.RecordBuilder
 	recordTypes    map[int]*zed.TypeRecord
 	table          map[string]*Row
 	limit          int
@@ -68,7 +69,7 @@ type Row struct {
 	reducers valRow
 }
 
-func NewAggregator(ctx context.Context, zctx *zed.Context, keyRefs, keyExprs, aggRefs []expr.Evaluator, aggs []*expr.Aggregator, builder *zed.ColumnBuilder, limit int, inputDir order.Direction, partialsIn, partialsOut bool) (*Aggregator, error) {
+func NewAggregator(ctx context.Context, zctx *zed.Context, keyRefs, keyExprs, aggRefs []expr.Evaluator, aggs []*expr.Aggregator, builder *zed.RecordBuilder, limit int, inputDir order.Direction, partialsIn, partialsOut bool) (*Aggregator, error) {
 	if limit == 0 {
 		limit = DefaultLimit
 	}
@@ -124,7 +125,7 @@ func New(pctx *op.Context, parent zbuf.Puller, keys []expr.Assignment, aggNames
 		names = append(names, e.LHS)
 	}
 	names = append(names, aggNames...)
-	builder, err := zed.NewColumnBuilder(pctx.Zctx, names)
+	builder, err := zed.NewRecordBuilder(pctx.Zctx, names)
 	if err != nil {
 		return nil, err
 	}
@@ -337,7 +338,7 @@ func (a *Aggregator) Consume(batch zbuf.Batch, this *zed.Value) error {
 	// We conveniently put the key type code at the end of the key string,
 	// so when we recontruct the key values below, we don't have skip over it.
 	keyType := a.keyTypes.Lookup(types)
-	keyBytes = zcode.AppendUvarint(keyBytes, uint64(keyType))
+	keyBytes = binary.AppendUvarint(keyBytes, uint64(keyType))
 	a.keyCache = keyBytes
 
 	row, ok := a.table[string(keyBytes)]
