@@ -7,6 +7,8 @@ import (
 
 	"github.com/brimdata/zed"
 	"github.com/brimdata/zed/compiler/ast/dag"
+	"github.com/brimdata/zed/order"
+	"github.com/brimdata/zed/pkg/field"
 	"github.com/brimdata/zed/runtime/expr"
 	"github.com/brimdata/zed/zio"
 )
@@ -14,6 +16,8 @@ import (
 type Filter interface {
 	AsEvaluator() (expr.Evaluator, error)
 	AsBufferFilter() (*expr.BufferFilter, error)
+	AsKeySpanFilter(field.Path, order.Which) (*expr.SpanFilter, error)
+	AsKeyCroppedByFilter(field.Path, order.Which) (*expr.SpanFilter, error)
 	//XXX This is here to break an import loop between lake and compiler.
 	// We will remove this in a subsequent PR when the runtime/sequence,vector
 	// packages will implement a pushdown language that is not based on dag.Expr
@@ -47,15 +51,22 @@ type Progress struct {
 	RecordsMatched int64 `zed:"records_matched" json:"records_matched"`
 }
 
+var _ Meter = (*Progress)(nil)
+
 // Add updates its receiver by adding to it the values in ss.
 func (p *Progress) Add(in Progress) {
-	atomic.AddInt64(&p.BytesRead, in.BytesRead)
-	atomic.AddInt64(&p.BytesMatched, in.BytesMatched)
-	atomic.AddInt64(&p.RecordsRead, in.RecordsRead)
-	atomic.AddInt64(&p.RecordsMatched, in.RecordsMatched)
+	if p != nil {
+		atomic.AddInt64(&p.BytesRead, in.BytesRead)
+		atomic.AddInt64(&p.BytesMatched, in.BytesMatched)
+		atomic.AddInt64(&p.RecordsRead, in.RecordsRead)
+		atomic.AddInt64(&p.RecordsMatched, in.RecordsMatched)
+	}
 }
 
 func (p *Progress) Copy() Progress {
+	if p == nil {
+		return Progress{}
+	}
 	return Progress{
 		BytesRead:      atomic.LoadInt64(&p.BytesRead),
 		BytesMatched:   atomic.LoadInt64(&p.BytesMatched),
@@ -64,10 +75,27 @@ func (p *Progress) Copy() Progress {
 	}
 }
 
+func (p *Progress) Progress() Progress {
+	return p.Copy()
+}
+
 var ScannerBatchSize = 100
 
 // NewScanner returns a Scanner for r that filters records by filterExpr and s.
+// If r implements fmt.Stringer, the scanner reports errors using a prefix of the
+// string returned by its String method.
 func NewScanner(ctx context.Context, r zio.Reader, filterExpr Filter) (Scanner, error) {
+	s, err := newScanner(ctx, r, filterExpr)
+	if err != nil {
+		return nil, err
+	}
+	if stringer, ok := r.(fmt.Stringer); ok {
+		s = NamedScanner(s, stringer.String())
+	}
+	return s, nil
+}
+
+func newScanner(ctx context.Context, r zio.Reader, filterExpr Filter) (Scanner, error) {
 	var sa ScannerAble
 	if zf, ok := r.(*File); ok {
 		sa, _ = zf.Reader.(ScannerAble)
@@ -126,9 +154,7 @@ func (s *scanner) Read() (*zed.Value, error) {
 		}
 		atomic.AddInt64(&s.progress.BytesMatched, int64(len(this.Bytes)))
 		atomic.AddInt64(&s.progress.RecordsMatched, 1)
-		// Copy the underlying buffer because the next call to
-		// s.reader.Read will overwrite it.
-		return this.Copy(), nil
+		return this, nil
 	}
 }
 
@@ -161,3 +187,28 @@ func (n *namedScanner) Pull(done bool) (Batch, error) {
 	}
 	return b, err
 }
+
+func MultiScanner(scanners ...Scanner) Scanner {
+	return &multiScanner{scanners: scanners}
+}
+
+type multiScanner struct {
+	scanners []Scanner
+	progress Progress
+}
+
+func (m *multiScanner) Pull(done bool) (Batch, error) {
+	for len(m.scanners) > 0 {
+		batch, err := m.scanners[0].Pull(done)
+		if batch != nil || err != nil {
+			return batch, err
+		}
+		m.progress.Add(m.scanners[0].Progress())
+		m.scanners = m.scanners[1:]
+	}
+	return nil, nil
+}
+
+func (m *multiScanner) Progress() Progress {
+	return m.progress.Copy()
+}
