@@ -33,9 +33,9 @@ type (
 		Elements []Value
 	}
 	Union struct {
-		Type     zed.Type
-		Selector int
-		Value    Value
+		Type  zed.Type
+		Tag   int
+		Value Value
 	}
 	Enum struct {
 		Type zed.Type
@@ -173,7 +173,7 @@ func (a Analyzer) typeCheck(cast, parent zed.Type) error {
 
 func (a Analyzer) enterTypeDef(zctx *zed.Context, name string, typ zed.Type) (*zed.TypeNamed, error) {
 	var named *zed.TypeNamed
-	if IsTypeName(name) {
+	if !isNumeric(name) {
 		var err error
 		if named, err = zctx.LookupTypeNamed(name, typ); err != nil {
 			return nil, err
@@ -184,10 +184,23 @@ func (a Analyzer) enterTypeDef(zctx *zed.Context, name string, typ zed.Type) (*z
 	return named, nil
 }
 
+func isNumeric(s string) bool {
+	for _, r := range s {
+		if !isDigit(r) {
+			return false
+		}
+	}
+	return true
+}
+
+func isDigit(r rune) bool {
+	return r >= '0' && r <= '9'
+}
+
 func (a Analyzer) convertAny(zctx *zed.Context, val astzed.Any, cast zed.Type) (Value, error) {
 	// If we're casting something to a union, then the thing inside needs to
 	// describe itself and we can convert the inner value to a union value when
-	// we know its type (so we can code the selector).
+	// we know its type (so we can code the tag).
 	if union, ok := zed.TypeUnder(cast).(*zed.TypeUnion); ok {
 		v, err := a.convertAny(zctx, val, nil)
 		if err != nil {
@@ -310,7 +323,7 @@ func (a Analyzer) convertFields(zctx *zed.Context, in []astzed.Field, cols []zed
 func lookupRecordType(zctx *zed.Context, fields []astzed.Field, vals []Value) (*zed.TypeRecord, error) {
 	columns := make([]zed.Column, 0, len(fields))
 	for k, f := range fields {
-		columns = append(columns, zed.Column{f.Name, vals[k].TypeOf()})
+		columns = append(columns, zed.Column{Name: f.Name, Type: vals[k].TypeOf()})
 	}
 	return zctx.LookupTypeRecord(columns)
 }
@@ -428,19 +441,19 @@ func (a Analyzer) convertSet(zctx *zed.Context, set *astzed.Set, cast zed.Type)
 func (a Analyzer) convertUnion(zctx *zed.Context, v Value, union *zed.TypeUnion, cast zed.Type) (Value, error) {
 	valType := v.TypeOf()
 	if valType == zed.TypeNull {
-		// Set selector to -1 to signal to the builder to encode a null.
+		// Set tag to -1 to signal to the builder to encode a null.
 		return &Union{
-			Type:     cast,
-			Selector: -1,
-			Value:    v,
+			Type:  cast,
+			Tag:   -1,
+			Value: v,
 		}, nil
 	}
 	for k, typ := range union.Types {
 		if valType == typ {
 			return &Union{
-				Type:     cast,
-				Selector: k,
-				Value:    v,
+				Type:  cast,
+				Tag:   k,
+				Value: v,
 			}, nil
 		}
 	}
@@ -634,7 +647,7 @@ func (a Analyzer) convertTypeRecord(zctx *zed.Context, typ *astzed.TypeRecord) (
 		if err != nil {
 			return nil, err
 		}
-		columns = append(columns, zed.Column{f.Name, typ})
+		columns = append(columns, zed.Column{Name: f.Name, Type: typ})
 	}
 	return zctx.LookupTypeRecord(columns)
 }
