@@ -12,6 +12,7 @@ import (
 	"github.com/brimdata/zed"
 	"github.com/brimdata/zed/pkg/nano"
 	"github.com/brimdata/zed/zcode"
+	"golang.org/x/exp/slices"
 )
 
 //XXX handle new TypeError => marshal as a ZSON string?
@@ -145,19 +146,6 @@ func (m *MarshalZNGContext) Marshal(v interface{}) (*zed.Value, error) {
 	return zed.NewValue(typ, it.Next()), nil
 }
 
-//XXX get rid of this?
-func (m *MarshalZNGContext) MarshalRecord(v interface{}) (*zed.Value, error) {
-	m.Builder.Reset()
-	typ, err := m.encodeValue(reflect.ValueOf(v))
-	if err != nil {
-		return nil, err
-	}
-	if !zed.IsRecordType(typ) {
-		return nil, errors.New("not a record")
-	}
-	return zed.NewValue(typ, m.Builder.Bytes().Body()), nil
-}
-
 func (m *MarshalZNGContext) MarshalCustom(names []string, fields []interface{}) (*zed.Value, error) {
 	if len(names) != len(fields) {
 		return nil, errors.New("fields and columns don't match")
@@ -169,7 +157,7 @@ func (m *MarshalZNGContext) MarshalCustom(names []string, fields []interface{})
 		if err != nil {
 			return nil, err
 		}
-		cols = append(cols, zed.Column{names[k], typ})
+		cols = append(cols, zed.Column{Name: names[k], Type: typ})
 	}
 	// XXX issue #1836
 	// Since this can be the inner loop here and nowhere else do we call
@@ -276,37 +264,15 @@ func (m *MarshalZNGContext) encodeValue(v reflect.Value) (zed.Type, error) {
 	if err != nil {
 		return nil, err
 	}
-	if m.decorator != nil || m.bindings != nil {
-		// Don't create named types for interface types as this is just
-		// one value for that interface and it's the underlying concrete
-		// types that implement the interface that we want to name.
-		if !v.IsValid() || v.Kind() == reflect.Interface {
-			return typ, nil
-		}
-		name := v.Type().Name()
-		kind := v.Kind().String()
-		if name != "" && name != kind {
-			// We do not want to further decorate nano.Ts as
-			// it's already been converted to a Zed time;
-			// likewise for zed.Value, which gets encoded as
-			// itself and its own named type if it has one.
-			if t := v.Type(); t == nanoTsType || t == zngValueType || t == netipAddrType || t == netIPType {
-				return typ, nil
-			}
-			path := v.Type().PkgPath()
-			var named string
-			if m.bindings != nil {
-				named = m.bindings[typeFull(name, path)]
-			}
-			if named == "" && m.decorator != nil {
-				named = m.decorator(name, path)
-			}
-			if named != "" {
-				return m.Context.LookupTypeNamed(named, typ)
-			}
-		}
+	if _, ok := typ.(*zed.TypeNamed); ok {
+		// We already have a named type.
+		return typ, nil
 	}
-	return typ, nil
+	if !v.IsValid() {
+		// v.Type will panic.
+		return typ, nil
+	}
+	return m.lookupTypeNamed(v.Type(), typ)
 }
 
 func (m *MarshalZNGContext) encodeAny(v reflect.Value) (zed.Type, error) {
@@ -329,6 +295,11 @@ func (m *MarshalZNGContext) encodeAny(v reflect.Value) (zed.Type, error) {
 		m.Builder.Append(zv.Bytes)
 		return typ, nil
 	}
+	if typ, ok := v.Interface().(zed.Type); ok {
+		val := m.Context.LookupTypeValue(typ)
+		m.Builder.Append(val.Bytes)
+		return val.Type, nil
+	}
 	if ip, ok := v.Interface().(net.IP); ok {
 		if a, err := netip.ParseAddr(ip.String()); err == nil {
 			m.Builder.Append(zed.EncodeIP(a))
@@ -491,7 +462,7 @@ func (m *MarshalZNGContext) encodeRecord(sval reflect.Value) (zed.Type, error) {
 		if err != nil {
 			return nil, err
 		}
-		columns = append(columns, zed.Column{name, typ})
+		columns = append(columns, zed.Column{Name: name, Type: typ})
 	}
 	m.Builder.EndContainer()
 	return m.Context.LookupTypeRecord(columns)
@@ -514,82 +485,108 @@ func (m *MarshalZNGContext) encodeArrayBytes(arrayVal reflect.Value) (zed.Type,
 }
 
 func (m *MarshalZNGContext) encodeArray(arrayVal reflect.Value) (zed.Type, error) {
-	len := arrayVal.Len()
 	m.Builder.BeginContainer()
-	var innerType zed.Type
-	for i := 0; i < len; i++ {
+	arrayLen := arrayVal.Len()
+	types := make([]zed.Type, 0, arrayLen)
+	for i := 0; i < arrayLen; i++ {
 		item := arrayVal.Index(i)
 		typ, err := m.encodeValue(item)
 		if err != nil {
 			return nil, err
 		}
-		// XXX See bug #2575
-		innerType = typ
+		types = append(types, typ)
 	}
-	m.Builder.EndContainer()
-	if innerType == nil {
+	uniqueTypes := zed.UniqueTypes(slices.Clone(types))
+	var innerType zed.Type
+	switch len(uniqueTypes) {
+	case 0:
 		// if slice was empty, look up the type without a value
 		var err error
 		innerType, err = m.lookupType(arrayVal.Type().Elem())
 		if err != nil {
 			return nil, err
 		}
+	case 1:
+		innerType = types[0]
+	default:
+		unionType := m.Context.LookupTypeUnion(uniqueTypes)
+		// Convert each container element to the union type.
+		m.Builder.TransformContainer(func(bytes zcode.Bytes) zcode.Bytes {
+			var b zcode.Builder
+			for i, it := 0, bytes.Iter(); !it.Done(); i++ {
+				zed.BuildUnion(&b, unionType.TagOf(types[i]), it.Next())
+			}
+			return b.Bytes()
+		})
+		innerType = unionType
 	}
+	m.Builder.EndContainer()
 	return m.Context.LookupTypeArray(innerType), nil
 }
 
-func (m *MarshalZNGContext) lookupType(typ reflect.Type) (zed.Type, error) {
-	switch typ.Kind() {
+func (m *MarshalZNGContext) lookupType(t reflect.Type) (zed.Type, error) {
+	var typ zed.Type
+	switch t.Kind() {
 	case reflect.Array, reflect.Slice:
-		if typ.Elem().Kind() == reflect.Uint8 {
-			return zed.TypeBytes, nil
+		if t.Elem().Kind() == reflect.Uint8 {
+			typ = zed.TypeBytes
+		} else {
+			inner, err := m.lookupType(t.Elem())
+			if err != nil {
+				return nil, err
+			}
+			typ = m.Context.LookupTypeArray(inner)
 		}
-		typ, err := m.lookupType(typ.Elem())
-		if err != nil {
-			return nil, err
-		}
-		return m.Context.LookupTypeArray(typ), nil
 	case reflect.Map:
-		key, err := m.lookupType(typ.Key())
+		key, err := m.lookupType(t.Key())
 		if err != nil {
 			return nil, err
 		}
-		val, err := m.lookupType(typ.Elem())
+		val, err := m.lookupType(t.Elem())
 		if err != nil {
 			return nil, err
 		}
-		return m.Context.LookupTypeMap(key, val), nil
+		typ = m.Context.LookupTypeMap(key, val)
 	case reflect.Struct:
-		return m.lookupTypeRecord(typ)
+		var err error
+		typ, err = m.lookupTypeRecord(t)
+		if err != nil {
+			return nil, err
+		}
 	case reflect.Ptr:
-		return m.lookupType(typ.Elem())
+		var err error
+		typ, err = m.lookupType(t.Elem())
+		if err != nil {
+			return nil, err
+		}
 	case reflect.String:
-		return zed.TypeString, nil
+		typ = zed.TypeString
 	case reflect.Bool:
-		return zed.TypeBool, nil
+		typ = zed.TypeBool
 	case reflect.Int, reflect.Int64:
-		return zed.TypeInt64, nil
+		typ = zed.TypeInt64
 	case reflect.Int32:
-		return zed.TypeInt32, nil
+		typ = zed.TypeInt32
 	case reflect.Int16:
-		return zed.TypeInt16, nil
+		typ = zed.TypeInt16
 	case reflect.Int8:
-		return zed.TypeInt8, nil
+		typ = zed.TypeInt8
 	case reflect.Uint, reflect.Uint64:
-		return zed.TypeUint64, nil
+		typ = zed.TypeUint64
 	case reflect.Uint32:
-		return zed.TypeUint32, nil
+		typ = zed.TypeUint32
 	case reflect.Uint16:
-		return zed.TypeUint16, nil
+		typ = zed.TypeUint16
 	case reflect.Uint8:
-		return zed.TypeUint8, nil
+		typ = zed.TypeUint8
 	case reflect.Float32:
-		return zed.TypeFloat32, nil
+		typ = zed.TypeFloat32
 	case reflect.Float64:
-		return zed.TypeFloat64, nil
+		typ = zed.TypeFloat64
 	default:
-		return nil, fmt.Errorf("unsupported type: %v", typ.Kind())
+		return nil, fmt.Errorf("unsupported type: %v", t.Kind())
 	}
+	return m.lookupTypeNamed(t, typ)
 }
 
 func (m *MarshalZNGContext) lookupTypeRecord(structType reflect.Type) (zed.Type, error) {
@@ -601,16 +598,54 @@ func (m *MarshalZNGContext) lookupTypeRecord(structType reflect.Type) (zed.Type,
 		if err != nil {
 			return nil, err
 		}
-		columns = append(columns, zed.Column{name, fieldType})
+		columns = append(columns, zed.Column{Name: name, Type: fieldType})
 	}
 	return m.Context.LookupTypeRecord(columns)
 }
 
+// lookupTypeNamed returns a named type for typ with a name derived from t.  It
+// returns typ if it shouldn't derive a name from t.
+func (m *MarshalZNGContext) lookupTypeNamed(t reflect.Type, typ zed.Type) (zed.Type, error) {
+	if m.decorator == nil && m.bindings == nil {
+		return typ, nil
+	}
+	// Don't create named types for interface types as this is just
+	// one value for that interface and it's the underlying concrete
+	// types that implement the interface that we want to name.
+	if t.Kind() == reflect.Interface {
+		return typ, nil
+	}
+	// We do not want to further decorate nano.Ts as
+	// it's already been converted to a Zed time;
+	// likewise for zed.Value, which gets encoded as
+	// itself and its own named type if it has one.
+	if t == nanoTsType || t == zngValueType || t == netipAddrType || t == netIPType {
+		return typ, nil
+	}
+	name := t.Name()
+	if name == "" || name == t.Kind().String() {
+		return typ, nil
+	}
+	path := t.PkgPath()
+	var named string
+	if m.bindings != nil {
+		named = m.bindings[typeFull(name, path)]
+	}
+	if named == "" && m.decorator != nil {
+		named = m.decorator(name, path)
+	}
+	if named == "" {
+		return typ, nil
+	}
+	return m.Context.LookupTypeNamed(named, typ)
+}
+
 type ZNGUnmarshaler interface {
 	UnmarshalZNG(*UnmarshalZNGContext, *zed.Value) error
 }
 
 type UnmarshalZNGContext struct {
+	zctx   *zed.Context
 	binder binder
 }
 
@@ -630,6 +665,12 @@ func incompatTypeError(zt zed.Type, v reflect.Value) error {
 	return fmt.Errorf("incompatible type translation: zng type %v go type %v go kind %v", FormatType(zt), v.Type(), v.Kind())
 }
 
+// SetContext provides an optional type context to the unmarshaler.  This is
+// needed only when unmarshaling Zed type values into Go zed.Type interface values.
+func (u *UnmarshalZNGContext) SetContext(zctx *zed.Context) {
+	u.zctx = zctx
+}
+
 func (u *UnmarshalZNGContext) Unmarshal(zv *zed.Value, v interface{}) error {
 	return u.decodeAny(zv, reflect.ValueOf(v))
 }
@@ -667,17 +708,25 @@ func (u *UnmarshalZNGContext) decodeAny(zv *zed.Value, v reflect.Value) error {
 	if !v.IsValid() {
 		return errors.New("cannot unmarshal into value provided")
 	}
-	if v.Type().Implements(unmarshalerTypeZNG) {
-		if v.IsNil() {
-			v.Set(reflect.New(v.Type().Elem()))
-		}
-		return v.Interface().(ZNGUnmarshaler).UnmarshalZNG(u, zv)
+	var m ZNGUnmarshaler
+	m, v = indirect(v, zv)
+	if m != nil {
+		return m.UnmarshalZNG(u, zv)
 	}
-	if v.CanAddr() {
-		pv := v.Addr()
-		if pv.CanInterface() && pv.Type().Implements(unmarshalerTypeZNG) {
-			return pv.Interface().(ZNGUnmarshaler).UnmarshalZNG(u, zv)
-		}
+	if _, ok := v.Interface().(zed.Value); ok {
+		// For zed.Values we simply set the reflect value to the
+		// zed.Value that has been decoded.
+		v.Set(reflect.ValueOf(*zv.Copy()))
+		return nil
+	}
+	if zed.TypeUnder(zv.Type) == zed.TypeNull {
+		// A zed null value should successfully unmarshal to any go type. Typed
+		// nulls however need to be type checked.
+		v.Set(reflect.Zero(v.Type()))
+		return nil
+	}
+	if v.Kind() == reflect.Pointer && zv.Bytes == nil {
+		return u.decodeNull(zv, v)
 	}
 	if _, ok := v.Interface().(nano.Ts); ok {
 		if zv.Type != zed.TypeTime {
@@ -690,19 +739,6 @@ func (u *UnmarshalZNGContext) decodeAny(zv *zed.Value, v reflect.Value) error {
 		v.Set(reflect.ValueOf(zed.DecodeTime(zv.Bytes)))
 		return nil
 	}
-	if _, ok := v.Interface().(zed.Value); ok {
-		// For zed.Values we simply set the reflect value to the
-		// zed.Value that has been decoded.
-		v.Set(reflect.ValueOf(*zv.Copy()))
-		return nil
-	}
-	if zed.TypeUnder(zv.Type) == zed.TypeNull && v.CanSet() {
-		// Since a zed TypeNull can resolve to any value go ahead an set the
-		// zero value if the value can be set. Otherwise the reflect type is
-		// probably a pointer and we'll take care of this below.
-		v.Set(reflect.Zero(v.Type()))
-		return nil
-	}
 	switch v.Kind() {
 	case reflect.Array:
 		return u.decodeArray(zv, v)
@@ -719,23 +755,36 @@ func (u *UnmarshalZNGContext) decodeAny(zv *zed.Value, v reflect.Value) error {
 		}
 		return u.decodeRecord(zv, v)
 	case reflect.Interface:
-		// This is an interface value.  If the underlying ZNG data has
-		// a type name (via a named type), then we'll see if there's a
-		// binding for it and unmarshal into an instance of Template
-		// in the binding.
-		typ, err := u.lookupType(zv.Type)
+		if zed.TypeUnder(zv.Type) == zed.TypeType {
+			if u.zctx == nil {
+				return errors.New("cannot unmarshal type value without type context")
+			}
+			typ, err := u.zctx.LookupByValue(zv.Bytes)
+			if err != nil {
+				return err
+			}
+			v.Set(reflect.ValueOf(typ))
+			return nil
+		}
+		// If the interface value isn't null, then the user has provided
+		// an underlying value to unmarshal into.  So we just recursively
+		// decode the value into this existing value and return.
+		if !v.IsNil() {
+			return u.decodeAny(zv, v.Elem())
+		}
+		template, err := u.lookupGoType(zv.Type, zv.Bytes)
 		if err != nil {
 			return err
 		}
-		if typ == nil {
-			// If typ is nil, then the value must be of ZNG type null
+		if template == nil {
+			// If the template is nil, then the value must be of ZNG type null
 			// and ZNG type values can only have value null.  So, we
 			// set it to null of the type given for the marshaled-into
 			// value and return.
 			v.Set(reflect.Zero(v.Type()))
 			return nil
 		}
-		concrete := reflect.New(typ)
+		concrete := reflect.New(template)
 		if err := u.decodeAny(zv, concrete.Elem()); err != nil {
 			return err
 		}
@@ -749,17 +798,6 @@ func (u *UnmarshalZNGContext) decodeAny(zv *zed.Value, v reflect.Value) error {
 			v.Set(concrete)
 		}
 		return nil
-	case reflect.Ptr:
-		for v.Kind() == reflect.Ptr {
-			if v.CanSet() && zv.Bytes == nil {
-				return u.decodeNull(zv, v)
-			}
-			if v.IsNil() {
-				v.Set(reflect.New(v.Type().Elem()))
-			}
-			v = v.Elem()
-		}
-		return u.decodeAny(zv, v)
 	case reflect.String:
 		// XXX We bundle string, type, error all into string.
 		// See issue #1853.
@@ -833,6 +871,42 @@ func (u *UnmarshalZNGContext) decodeAny(zv *zed.Value, v reflect.Value) error {
 	}
 }
 
+// Adapted from:
+// https://github.com/golang/go/blob/46ab7a5c4f80d912f25b6b3e1044282a2a79df8b/src/encoding/json/decode.go#L426
+func indirect(v reflect.Value, zv *zed.Value) (ZNGUnmarshaler, reflect.Value) {
+	var nilptr reflect.Value
+	// If v is a named type and is addressable,
+	// start with its address, so that if the type has pointer methods,
+	// we find them.
+	if v.Kind() != reflect.Pointer && v.Type().Name() != "" && v.CanAddr() {
+		v = v.Addr()
+	}
+	for {
+		if v.Kind() != reflect.Pointer {
+			break
+		}
+		if v.CanSet() && zv.Bytes == nil {
+			// If the reflect value can be set and the zed Value is nil we want
+			// to store this pointer since if destination is not a zed.Value the
+			// pointer will be set to nil.
+			nilptr = v
+		}
+		if v.IsNil() {
+			v.Set(reflect.New(v.Type().Elem()))
+		}
+		if v.Type().NumMethod() > 0 && v.CanInterface() {
+			if u, ok := v.Interface().(ZNGUnmarshaler); ok {
+				return u, reflect.Value{}
+			}
+		}
+		v = v.Elem()
+	}
+	if _, ok := v.Interface().(zed.Value); !ok && nilptr.IsValid() {
+		return nil, nilptr
+	}
+	return nil, v
+}
+
 func (u *UnmarshalZNGContext) decodeNull(zv *zed.Value, v reflect.Value) error {
 	inner := v
 	for inner.Kind() == reflect.Ptr {
@@ -906,9 +980,13 @@ func (u *UnmarshalZNGContext) decodeMap(zv *zed.Value, mapVal reflect.Value) err
 }
 
 func (u *UnmarshalZNGContext) decodeRecord(zv *zed.Value, sval reflect.Value) error {
+	if union, ok := zv.Type.(*zed.TypeUnion); ok {
+		typ, bytes := union.Untag(zv.Bytes)
+		zv = zed.NewValue(typ, bytes)
+	}
 	recType, ok := zed.TypeUnder(zv.Type).(*zed.TypeRecord)
 	if !ok {
-		return fmt.Errorf("cannot unmarshal Zed type %q into Go struct", FormatType(zv.Type))
+		return fmt.Errorf("cannot unmarshal Zed value %q into Go struct", String(zv))
 	}
 	nameToField := make(map[string]int)
 	stype := sval.Type()
@@ -949,7 +1027,7 @@ func (u *UnmarshalZNGContext) decodeArray(zv *zed.Value, arrVal reflect.Value) e
 	}
 	arrType, ok := typ.(*zed.TypeArray)
 	if !ok {
-		return errors.New("not an array")
+		return fmt.Errorf("unmarshaling type %q: not an array", String(typ))
 	}
 	if zv.Bytes == nil {
 		// XXX The inner type of the null should be checked.
@@ -1083,7 +1161,11 @@ func typeNameOfValue(value interface{}) (string, error) {
 	return fmt.Sprintf("%s.%s", typ.PkgPath(), typ.Name()), nil
 }
 
-func (u *UnmarshalZNGContext) lookupType(typ zed.Type) (reflect.Type, error) {
+// lookupGoType builds a Go type for the Zed value given by typ and bytes.
+// This process requires
+// a value rather than a Zed type as it must determine the types of union elements
+// from their tags.
+func (u *UnmarshalZNGContext) lookupGoType(typ zed.Type, bytes zcode.Bytes) (reflect.Type, error) {
 	switch typ := typ.(type) {
 	case *zed.TypeNamed:
 		if template := u.binder.lookup(typ.Name); template != nil {
@@ -1095,33 +1177,71 @@ func (u *UnmarshalZNGContext) lookupType(typ zed.Type) (reflect.Type, error) {
 		// by reflect when the Set() method is called on the
 		// value and the concrete value doesn't implement the
 		// interface.
-		return u.lookupType(typ.Type)
+		return u.lookupGoType(typ.Type, bytes)
 	case *zed.TypeRecord:
-		return u.lookupTypeRecord(typ)
+		return nil, errors.New("unmarshaling records into interface value requires type binding")
 	case *zed.TypeArray:
-		elemType, err := u.lookupType(typ.Type)
+		// If we got here, we know the array type wasn't named and
+		// therefore cannot have mixed-type elements.  So we don't need
+		// to traverse the array and can just take the first element
+		// as the template value to recurse upon.  If there are actually
+		// heterogenous values, then the Go reflect package will raise
+		// the problem when decoding the value.
+		// If the inner type is a union, it must be a named-type union
+		// so we know what Go type to use as the elements of the array,
+		// which obviously can only be interface values for mixed types.
+		// XXX there's a corner case here for union type where all the
+		// elements of the array have the same tag, in which case you
+		// can have a normal array of that tag's type.
+		// We let the reflect package catch errors where the array contents
+		// are not consistent.  All we need to do here is make sure the
+		// interface name is in the bindings and the elemType will be
+		// the appropriate interface type.
+		it := bytes.Iter()
+		if it.Done() {
+			bytes = nil
+		} else {
+			bytes = it.Next()
+		}
+		elemType, err := u.lookupGoType(typ.Type, bytes)
 		if err != nil {
 			return nil, err
 		}
 		return reflect.SliceOf(elemType), nil
 	case *zed.TypeSet:
-		elemType, err := u.lookupType(typ.Type)
+		// See comment above for TypeArray as it applies here.
+		it := bytes.Iter()
+		if it.Done() {
+			bytes = nil
+		} else {
+			bytes = it.Next()
+		}
+		elemType, err := u.lookupGoType(typ.Type, bytes)
 		if err != nil {
 			return nil, err
 		}
 		return reflect.SliceOf(elemType), nil
-	case *zed.TypeUnion, *zed.TypeEnum:
+	case *zed.TypeUnion:
+		return u.lookupGoType(typ.Untag(bytes))
+	case *zed.TypeEnum:
 		// For now just return nil here. The layer above will flag
 		// a type error.  At some point, we can create Go-native data structures
 		// in package zng for representing a union or enum as a standalone
 		// entity.  See issue #1853.
 		return nil, nil
 	case *zed.TypeMap:
-		keyType, err := u.lookupType(typ.KeyType)
+		it := bytes.Iter()
+		if it.Done() {
+			return nil, fmt.Errorf("corrupt Zed map value in Zed unmarshal: type %q", String(typ))
+		}
+		keyType, err := u.lookupGoType(typ.KeyType, it.Next())
 		if err != nil {
 			return nil, err
 		}
-		valType, err := u.lookupType(typ.ValType)
+		if it.Done() {
+			return nil, fmt.Errorf("corrupt Zed map value in Zed unmarshal: type %q", String(typ))
+		}
+		valType, err := u.lookupGoType(typ.ValType, it.Next())
 		if err != nil {
 			return nil, err
 		}
@@ -1131,10 +1251,6 @@ func (u *UnmarshalZNGContext) lookupType(typ zed.Type) (reflect.Type, error) {
 	}
 }
 
-func (u *UnmarshalZNGContext) lookupTypeRecord(typ *zed.TypeRecord) (reflect.Type, error) {
-	return nil, errors.New("unmarshaling records into interface value requires type binding")
-}
-
 func (u *UnmarshalZNGContext) lookupPrimitiveType(typ zed.Type) (reflect.Type, error) {
 	var v interface{}
 	switch typ := typ.(type) {
