@@ -25,7 +25,7 @@ func toZSON(t *testing.T, rec *zed.Value) string {
 }
 
 func boomerang(t *testing.T, in interface{}, out interface{}) {
-	rec, err := zson.NewZNGMarshaler().MarshalRecord(in)
+	rec, err := zson.NewZNGMarshaler().Marshal(in)
 	require.NoError(t, err)
 	var buf bytes.Buffer
 	zw := zngio.NewWriter(zio.NopCloser(&buf))
@@ -51,7 +51,7 @@ func TestMarshalZNG(t *testing.T) {
 		Sub1    S2
 		PField1 *bool
 	}
-	rec, err := zson.NewZNGMarshaler().MarshalRecord(S1{
+	rec, err := zson.NewZNGMarshaler().Marshal(S1{
 		Field1: "value1",
 		Sub1: S2{
 			Field2: "value2",
@@ -93,20 +93,29 @@ type ZNGThings struct {
 }
 
 func TestMarshalSlice(t *testing.T) {
+	m := zson.NewZNGMarshaler()
+	m.Decorate(zson.StyleSimple)
+
 	s := []ZNGThing{{"hello", 123}, {"world", 0}}
 	r := ZNGThings{s}
-	m := zson.NewZNGMarshaler()
-	rec, err := m.MarshalRecord(r)
+	rec, err := m.Marshal(r)
 	require.NoError(t, err)
 	require.NotNil(t, rec)
-	assert.Equal(t, `{Things:[{a:"hello",B:123},{a:"world",B:0}]}`, toZSON(t, rec))
+	assert.Equal(t, `{Things:[{a:"hello",B:123}(=ZNGThing),{a:"world",B:0}(ZNGThing)]}(=ZNGThings)`,
+		toZSON(t, rec))
 
 	empty := []ZNGThing{}
 	r2 := ZNGThings{empty}
-	rec2, err := m.MarshalRecord(r2)
+	rec2, err := m.Marshal(r2)
 	require.NoError(t, err)
 	require.NotNil(t, rec2)
-	assert.Equal(t, "{Things:[]([{a:string,B:int64}])}", toZSON(t, rec2))
+	assert.Equal(t, "{Things:[]([ZNGThing={a:string,B:int64}])}(=ZNGThings)", toZSON(t, rec2))
+
+	rec3, err := m.Marshal(ZNGThings{nil})
+	require.NoError(t, err)
+	require.NotNil(t, rec3)
+	assert.Equal(t, "{Things:null([ZNGThing={a:string,B:int64}])}(=ZNGThings)", toZSON(t, rec3))
+
 }
 
 func TestMarshalNilSlice(t *testing.T) {
@@ -149,7 +158,7 @@ func TestIPType(t *testing.T) {
 	s := TestIP{Addr: netip.MustParseAddr("192.168.1.1")}
 	zctx := zed.NewContext()
 	m := zson.NewZNGMarshalerWithContext(zctx)
-	rec, err := m.MarshalRecord(s)
+	rec, err := m.Marshal(s)
 	require.NoError(t, err)
 	require.NotNil(t, rec)
 
@@ -176,7 +185,7 @@ func TestUnmarshalRecord(t *testing.T) {
 	v1 := T1{
 		T1f1: &T2{T2f1: T3{T3f1: 1, T3f2: 1.0}, T2f2: "t2f2-string1"},
 	}
-	rec, err := zson.NewZNGMarshaler().MarshalRecord(v1)
+	rec, err := zson.NewZNGMarshaler().Marshal(v1)
 	require.NoError(t, err)
 	require.NotNil(t, rec)
 
@@ -208,7 +217,7 @@ func TestUnmarshalNull(t *testing.T) {
 		require.NoError(t, zson.UnmarshalZNG(zed.Null, &slice))
 		assert.Nil(t, slice)
 		slice = []int{1}
-		assert.EqualError(t, zson.UnmarshalZNG(zed.NullInt64, &slice), "not an array")
+		assert.EqualError(t, zson.UnmarshalZNG(zed.NullInt64, &slice), `unmarshaling type "int64": not an array`)
 		slice = []int{1}
 		v := zson.MustParseValue(zed.NewContext(), "null([int64])")
 		require.NoError(t, zson.UnmarshalZNG(v, &slice))
@@ -245,11 +254,18 @@ func TestUnmarshalNull(t *testing.T) {
 		var obj struct {
 			Test *testobj `zed:"test"`
 		}
-		val := zson.MustParseValue(zed.NewContext(), "{test: null({Val:int64})}")
+		val := zson.MustParseValue(zed.NewContext(), "{test:null({Val:int64})}")
 		require.NoError(t, zson.UnmarshalZNG(val, &obj))
 		require.Nil(t, obj.Test)
-		val = zson.MustParseValue(zed.NewContext(), "{test: null(ip)}")
-		require.EqualError(t, zson.UnmarshalZNG(val, &obj), "cannot unmarshal Zed type \"ip\" into Go struct")
+		val = zson.MustParseValue(zed.NewContext(), "{test:null(ip)}")
+		require.EqualError(t, zson.UnmarshalZNG(val, &obj), `cannot unmarshal Zed value "null(ip)" into Go struct`)
+		var slice struct {
+			Test []string `zed:"test"`
+		}
+		slice.Test = []string{"1"}
+		val = zson.MustParseValue(zed.NewContext(), "{test:null}")
+		require.NoError(t, zson.UnmarshalZNG(val, &slice))
+		require.Nil(t, slice.Test)
 	})
 }
 
@@ -261,7 +277,7 @@ func TestUnmarshalSlice(t *testing.T) {
 		T1f1: []bool{true, false, true},
 	}
 	zctx := zed.NewContext()
-	rec, err := zson.NewZNGMarshalerWithContext(zctx).MarshalRecord(v1)
+	rec, err := zson.NewZNGMarshalerWithContext(zctx).Marshal(v1)
 	require.NoError(t, err)
 	require.NotNil(t, rec)
 
@@ -278,7 +294,7 @@ func TestUnmarshalSlice(t *testing.T) {
 		Field1: []*int{intp(1), intp(2)},
 	}
 	zctx = zed.NewContext()
-	rec, err = zson.NewZNGMarshalerWithContext(zctx).MarshalRecord(v3)
+	rec, err = zson.NewZNGMarshalerWithContext(zctx).Marshal(v3)
 	require.NoError(t, err)
 	require.NotNil(t, rec)
 
@@ -314,7 +330,7 @@ func TestMarshalInterface(t *testing.T) {
 	}
 	m1 := testMarshaler("m1")
 	r1 := rectype{M1: &m1, M2: testMarshaler("m2")}
-	rec, err := zson.NewZNGMarshaler().MarshalRecord(r1)
+	rec, err := zson.NewZNGMarshaler().Marshal(r1)
 	require.NoError(t, err)
 	require.NotNil(t, rec)
 	assert.Equal(t, `{M1:"marshal-m1",M2:"marshal-m2"}`, toZSON(t, rec))
@@ -334,7 +350,7 @@ func TestMarshalArray(t *testing.T) {
 	}
 	a2 := &[2]string{"foo", "bar"}
 	r1 := rectype{A1: [2]int8{1, 2}, A2: a2} // A3 left as nil
-	rec, err := zson.NewZNGMarshaler().MarshalRecord(r1)
+	rec, err := zson.NewZNGMarshaler().Marshal(r1)
 	require.NoError(t, err)
 	require.NotNil(t, rec)
 	const expected = `{A1:[1(int8),2(int8)],A2:["foo","bar"],A3:null([bytes])}`
@@ -377,7 +393,7 @@ func TestNumbers(t *testing.T) {
 		F32:  math.MaxFloat32,
 		F64:  math.MaxFloat64,
 	}
-	rec, err := zson.NewZNGMarshaler().MarshalRecord(r1)
+	rec, err := zson.NewZNGMarshaler().Marshal(r1)
 	require.NoError(t, err)
 	require.NotNil(t, rec)
 	const expected = "{I:-9223372036854775808,I8:-128(int8),I16:-32768(int16),I32:-2147483648(int32),I64:-9223372036854775808,U:18446744073709551615(uint64),UI8:255(uint8),UI16:65535(uint16),UI32:4294967295(uint32),UI64:18446744073709551615(uint64),F32:3.4028235e+38(float32),F64:1.7976931348623157e+308}"
@@ -448,7 +464,7 @@ func TestInterfaceZNGMarshal(t *testing.T) {
 	m.Decorate(zson.StyleFull)
 	zv, err = m.Marshal(rolls)
 	require.NoError(t, err)
-	assert.Equal(t, "github.com/brimdata/zed/zson_test.Rolls=[int64]", zson.String(zv.Type))
+	assert.Equal(t, `"github.com/brimdata/zed/zson_test.Rolls"=[int64]`, zson.String(zv.Type))
 
 	plain := []int32{1, 2, 3}
 	zv, err = m.Marshal(plain)
@@ -474,7 +490,7 @@ func TestInterfaceUnmarshal(t *testing.T) {
 
 	var thingI interface{}
 	err = u.Unmarshal(zv, &thingI)
-	require.NoError(t, err)
+	require.NoError(t, err, zson.String(zv))
 	actualThing, ok := thingI.(*ZNGThing)
 	assert.Equal(t, true, ok)
 	assert.Equal(t, t1, actualThing)
@@ -483,7 +499,7 @@ func TestInterfaceUnmarshal(t *testing.T) {
 	var genericThing interface{}
 	err = u2.Unmarshal(zv, &genericThing)
 	require.Error(t, err)
-	assert.Equal(t, "unmarshaling records into interface value requires type binding", err.Error())
+	assert.Equal(t, `unmarshaling records into interface value requires type binding`, err.Error())
 }
 
 func TestBindings(t *testing.T) {
@@ -602,3 +618,34 @@ func TestMultipleZedValues(t *testing.T) {
 	assert.Equal(t, "foo", string(foo.Bytes))
 	assert.Equal(t, "bar", string(bar.Bytes))
 }
+
+func TestZedValues(t *testing.T) {
+	test := func(t *testing.T, name, s string, v interface{}) {
+		t.Run(name, func(t *testing.T) {
+			val := zson.MustParseValue(zed.NewContext(), s)
+			err := zson.UnmarshalZNG(val, v)
+			require.NoError(t, err)
+			val, err = zson.MarshalZNG(v)
+			require.NoError(t, err)
+			assert.Equal(t, s, zson.MustFormatValue(val))
+		})
+	}
+	var testptr struct {
+		Value *zed.Value `zed:"value"`
+	}
+	t.Run("pointer", func(t *testing.T) {
+		test(t, "string", "{value:\"foo\"}", &testptr)
+		test(t, "typed-null", "{value:null(time)}", &testptr)
+		test(t, "null", "{value:null}", &testptr)
+		test(t, "record", "{value:{foo:1,bar:\"baz\"}}", &testptr)
+	})
+	var teststruct struct {
+		Value zed.Value `zed:"value"`
+	}
+	t.Run("struct", func(t *testing.T) {
+		test(t, "string", "{value:\"foo\"}", &teststruct)
+		test(t, "typed-null", "{value:null(time)}", &teststruct)
+		test(t, "null", "{value:null}", &teststruct)
+		test(t, "record", "{value:{foo:1,bar:\"baz\"}}", &teststruct)
+	})
+}
