@@ -1,7 +1,6 @@
 package zson
 
 import (
-	"strconv"
 	"unicode"
 )
 
@@ -11,7 +10,7 @@ func IsIdentifier(s string) bool {
 	}
 	first := true
 	for _, c := range s {
-		if !idChar(c) && (first || !unicode.IsDigit(c)) {
+		if !idChar(c) && (first || !isDigit(c)) {
 			return false
 		}
 		first = false
@@ -23,18 +22,19 @@ func idChar(c rune) bool {
 	return unicode.IsLetter(c) || c == '_' || c == '$'
 }
 
-// IsTypeName returns true iff s is a valid zson typedef name (exclusive
-// of integer names for locally-scoped typedefs).
+// IsTypeName returns true iff s is a valid, unquoted ZSON type name.
 func IsTypeName(s string) bool {
-	for _, c := range s {
+	for k, c := range s {
 		if !typeChar(c) {
 			return false
 		}
+		if k == 0 && isDigit(c) {
+			return false
+		}
 	}
-	_, err := strconv.ParseInt(s, 10, 64)
-	return err != nil
+	return true
 }
 
 func typeChar(c rune) bool {
-	return idChar(c) || unicode.IsDigit(c) || c == '/' || c == '.'
+	return idChar(c) || isDigit(c) || c == '.'
 }
